@@ -45,7 +45,8 @@ class DiagramDrawItem : public DiagramItem
 {
 public:
     enum { Type = UserType + 16 };
-    enum DiagramType { Ellipse, Rectangle, Circle, RoundedRect, Rhombus, Triangle, DA , OTA, Note, Pie, MUX, DEMUX};
+    enum DiagramType { Ellipse, Rectangle, Circle, RoundedRect,
+                       Rhombus, Triangle, DA , OTA, Note, Pie, MUX, DEMUX, Square};
 
     DiagramDrawItem(DiagramType diagramType, QMenu *contextMenu,
         QGraphicsItem *parent = 0);
@@ -79,6 +80,9 @@ public:
     void setStartPoint(const QPointF pt);
     void setEndPoint(const QPointF pt);
 
+    void setPartnerItem(DiagramDrawItem *parterItem);
+    DiagramDrawItem *partnerItem();
+
 protected:
     void contextMenuEvent(QGraphicsSceneContextMenuEvent *event) override;
     QVariant itemChange(GraphicsItemChange change, const QVariant &value) override;
@@ -107,6 +111,7 @@ private:
     qreal myRadius;
     QPointF mStartPoint,mEndPoint;
     Rect mRect;
+    DiagramDrawItem *m_partnerItem;
 };
 
 #endif // DIAGRAMDRAWITEM_H
