@@ -97,6 +97,8 @@ DiagramScene::DiagramScene(QMenu *itemMenu, QObject *parent)
     myCursor.setZValue(10.0);
     myCursor.setFlag(QGraphicsItem::ItemIgnoresTransformations);
     addItem(&myCursor);
+
+    connect(this,&DiagramScene::selectionChanged,this,&DiagramScene::itemSelectionChangedSlot);
 }
 
 void DiagramScene::setLineColor(const QColor &color)
@@ -217,9 +219,9 @@ void DiagramScene::setMode(DiagramScene::Mode mode, bool m_abort)
 
     myMode = mode;
     switch (mode) {
-    case MoveItem:
     case MoveItems:
     case CopyItem:
+    case MoveItem:
         enableAllItems(true);
         break;
     default:
@@ -230,9 +232,11 @@ void DiagramScene::setMode(DiagramScene::Mode mode, bool m_abort)
 
 void DiagramScene::enableAllItems(bool enable)
 {
+    m_blockSelectionChanged=true;
     foreach(QGraphicsItem* item,items()){
         item->setEnabled(enable);
     }
+    m_blockSelectionChanged=!enable;
 }
 /*!
  * \brief make a text item as child of item
@@ -251,7 +255,7 @@ DiagramTextItem *DiagramScene::makeTextItem(QGraphicsItem *item)
     connect(textItem, &DiagramTextItem::receivedFocus,
             this, &DiagramScene::editorReceivedFocus);
     connect(textItem, &DiagramTextItem::selectedChange,
-            this, &DiagramScene::itemSelected);
+            this, &DiagramScene::textItemSelected);
     //addItem(textItem);
     textItem->setParentItem(item);
     textItem->setDefaultTextColor(myTextColor);
@@ -316,6 +320,7 @@ QList<QGraphicsItem *> DiagramScene::copyItems(QList<QGraphicsItem *> source)
         QGraphicsItem *insItem=copy(item);
         if(!insItem) continue;
         copiedItems.append(item);
+        item->setFlag(QGraphicsItem::ItemIsMovable,true);
         addItem(insItem);
         insItem->setPos(item->pos());
         insItem->setZValue(item->zValue());
@@ -368,6 +373,62 @@ qreal DiagramScene::getMinZ(QList<QGraphicsItem *> source)
     }
     return result;
 }
+/*!
+ * \brief check if item has partner and replace it with partner
+ * partner is used to place a copy on top of the scene to have visible handlers
+ * The principal item is the partner item
+ * \param item
+ */
+void DiagramScene::getPartneredItem(QGraphicsItem *&item) const
+{
+    // check if partner exists, use that instead
+    auto *pathItem=qgraphicsitem_cast<DiagramPathItem*>(item);
+    if(pathItem && pathItem->partnerItem()){
+        item=pathItem->partnerItem();
+    }
+    auto *splineItem=qgraphicsitem_cast<DiagramSplineItem*>(item);
+    if(splineItem && splineItem->partnerItem()){
+        item=splineItem->partnerItem();
+    }
+    auto *drawItem=qgraphicsitem_cast<DiagramDrawItem*>(item);
+    if(drawItem && drawItem->partnerItem()){
+        item=drawItem->partnerItem();
+    }
+}
+
+/*!
+ * \brief check if item is locked
+ * \param item
+ * \return
+ */
+bool DiagramScene::isItemLocked(QGraphicsItem *item)
+{
+    DiagramItem *diagramItem = qgraphicsitem_cast<DiagramItem*>(item);
+    if(diagramItem){
+        return diagramItem->isLocked();
+    }
+    DiagramPathItem *pathItem = qgraphicsitem_cast<DiagramPathItem*>(item);
+    if(pathItem){
+        return pathItem->isLocked();
+    }
+    DiagramSplineItem *splineItem = qgraphicsitem_cast<DiagramSplineItem*>(item);
+    if(splineItem){
+        return splineItem->isLocked();
+    }
+    return false;
+}
+/*!
+ * \brief remove partner item
+ * remove m_SelectedItem properly
+ */
+void DiagramScene::removePartnerItem()
+{
+    m_blockSelectionChanged=true;
+    this->removeItem(m_SelectedItem);
+    delete m_SelectedItem;
+    m_SelectedItem = nullptr;
+    m_blockSelectionChanged=false;
+}
 /*!
  * \brief filter selected child items
  * If in the list parent and child are selected, child is removed from list to avoid
@@ -377,7 +438,7 @@ qreal DiagramScene::getMinZ(QList<QGraphicsItem *> source)
 void DiagramScene::filterSelectedChildItems(QList<QGraphicsItem *> &lst)
 {
     foreach(QGraphicsItem* item,lst){
-        if(!item->flags().testFlag(QGraphicsItem::ItemIsMovable)){
+        if(isItemLocked(item)){
             item->setSelected(false);
             lst.removeOne(item);
             continue; // skip locked items
@@ -388,6 +449,7 @@ void DiagramScene::filterSelectedChildItems(QList<QGraphicsItem *> &lst)
                 lst.removeOne(item);
             }
         }
+        item->setFlag(QGraphicsItem::ItemIsMovable,true);
     }
 }
 /*!
@@ -442,6 +504,24 @@ bool DiagramScene::replaceText(const QString find_text, const QString replace_te
     }
     return success;
 }
+/*!
+ * \brief return selected items
+ * Replace items which have partners
+ * \return
+ */
+QList<QGraphicsItem *> DiagramScene::selectedItems(bool includeSelected)
+{
+    QList<QGraphicsItem *> lst = QGraphicsScene::selectedItems();
+
+    // check for partnered items
+    for(QGraphicsItem* &item:lst){
+        getPartneredItem(item);
+    }
+    if(includeSelected && m_SelectedItem){
+        lst.append(m_SelectedItem);
+    }
+    return lst;
+}
 
 void DiagramScene::setItemType(DiagramItem::DiagramType type)
 {
@@ -640,7 +720,7 @@ void DiagramScene::mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent)
         connect(textItem, &DiagramTextItem::receivedFocus,
                 this, &DiagramScene::editorReceivedFocus);
         connect(textItem, &DiagramTextItem::selectedChange,
-                this, &DiagramScene::itemSelected);
+                this, &DiagramScene::textItemSelected);
         addItem(textItem);
         textItem->setDefaultTextColor(myTextColor);
         textItem->setCorrectedPos(onGrid(mouseEvent->scenePos()));
@@ -758,12 +838,18 @@ void DiagramScene::mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent)
                 // speichere Referenzpunkt
                 myDx=point.rx();
                 myDy=point.ry();
+                // disable partner
+                if(m_SelectedItem){
+                    removePartnerItem();
+                }
             }
         }
         if(middleButton){
             // switch toolbar !!
             emit abortSignal();
         }
+        mouseEvent->accept();
+        return;
         break;
     }
     case CopyItem:
@@ -792,6 +878,8 @@ void DiagramScene::mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent)
             // switch toolbar !!
             emit abortSignal();
         }
+        mouseEvent->accept();
+        return;
         break;
     case CopyingItem:
         if (copiedItems.count() > 0){
@@ -807,6 +895,8 @@ void DiagramScene::mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent)
             // switch toolbar !!
             emit abortSignal();
         }
+        mouseEvent->accept();
+        return;
         break;
     case Zoom:
         startPoint=mouseEvent->scenePos();
@@ -842,6 +932,23 @@ void DiagramScene::mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent)
         QGraphicsScene::mouseMoveEvent(mouseEvent);
         if(mouseEvent->buttons()==Qt::LeftButton && mouseGrabberItem()){
             checkOnGrid();
+            if(m_SelectedItem){
+                DiagramDrawItem *drawItem=qgraphicsitem_cast<DiagramDrawItem*>(m_SelectedItem);
+                if(drawItem){
+                    DiagramDrawItem *partnerItem=drawItem->partnerItem();
+                    partnerItem->setPos(drawItem->pos());
+                }
+                DiagramPathItem *pathItem=qgraphicsitem_cast<DiagramPathItem*>(m_SelectedItem);
+                if(pathItem){
+                    DiagramPathItem *partnerItem=pathItem->partnerItem();
+                    partnerItem->setPos(pathItem->pos());
+                }
+                DiagramSplineItem *splineItem=qgraphicsitem_cast<DiagramSplineItem*>(m_SelectedItem);
+                if(splineItem){
+                    DiagramSplineItem *partnerItem=splineItem->partnerItem();
+                    partnerItem->setPos(splineItem->pos());
+                }
+            }
         }
         break;
     case MoveItems:
@@ -968,6 +1075,11 @@ void DiagramScene::mouseReleaseEvent(QGraphicsSceneMouseEvent *mouseEvent)
         }
         abort(true);
     }
+    if(myMode== CopyingItem){
+        mouseEvent->accept();
+        return;
+    }
+
     QGraphicsScene::mouseReleaseEvent(mouseEvent);
 }
 
@@ -997,6 +1109,7 @@ void DiagramScene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *mouseEvent)
                     textItem->setFocus();
                 }else{
                     // draw item, add text in center
+                    m_blockSelectionChanged=true;
                     textItem = makeTextItem(item);
                     DiagramDrawItem *drawItem=dynamic_cast<DiagramDrawItem *>(item);
                     if(drawItem && drawItem->diagramType()==DiagramDrawItem::Note){
@@ -1009,6 +1122,11 @@ void DiagramScene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *mouseEvent)
                     }
 
                     emit textInserted(textItem);
+                    // deselect drawitem
+                    item->setSelected(false);
+                    // remove partner
+                    removePartnerItem();
+                    m_blockSelectionChanged=false;
                 }
                 mouseEvent->accept();
                 break;
@@ -1025,6 +1143,7 @@ void DiagramScene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *mouseEvent)
                     p->update();
                 }else{
                     // path item, add text in center of line segment
+                    m_blockSelectionChanged=true;
                     textItem = makeTextItem(item);
                     textItem->setAlignment(Qt::AlignBottom|Qt::AlignHCenter);
                     // find correct position for tex
@@ -1059,6 +1178,7 @@ void DiagramScene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *mouseEvent)
                     p->update();
 
                     emit textInserted(textItem);
+                    m_blockSelectionChanged=false;
                 }
                 mouseEvent->accept();
                 break;
@@ -1080,7 +1200,7 @@ void DiagramScene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *mouseEvent)
 
 void DiagramScene::checkOnGrid()
 {
-    foreach (QGraphicsItem *item, selectedItems()) {
+    foreach (QGraphicsItem *item, selectedItems(true)) {
         if(item->parentItem()) continue; // don't change elements which are bound to other items, e.g. text for rectangle
         if(item->type()==DiagramTextItem::Type){
             DiagramTextItem *textItem=qgraphicsitem_cast<DiagramTextItem *>(item);
@@ -1124,7 +1244,7 @@ QGraphicsItem* DiagramScene::copy(QGraphicsItem* item)
         connect(textItem, &DiagramTextItem::receivedFocus,
                 this, &DiagramScene::editorReceivedFocus);
         connect(textItem, &DiagramTextItem::selectedChange,
-                this, &DiagramScene::itemSelected);
+                this, &DiagramScene::textItemSelected);
         return qgraphicsitem_cast<QGraphicsItem*>(textItem);
     }
         break;
@@ -1143,8 +1263,10 @@ QGraphicsItem* DiagramScene::copy(QGraphicsItem* item)
         break;
     }
     case DiagramSplineItem::Type:
+    {
         return qgraphicsitem_cast<QGraphicsItem*>(qgraphicsitem_cast<DiagramSplineItem*>(item)->copy());
         break;
+    }
     case QGraphicsItemGroup::Type:
     {
         QPointF p=item->pos();
@@ -1210,8 +1332,13 @@ void DiagramScene::copyToBuffer()
     }
 }
 
-void DiagramScene::pasteFromBuffer()
+void DiagramScene::pasteFromBuffer(QByteArray buffer)
 {
+    if(!buffer.isEmpty()){
+        // read from buffer
+        QJsonDocument doc=QJsonDocument::fromJson(buffer);
+        bufferedItems=read_in_json(doc,false);
+    }
     copiedItems.clear();
     selectedItems().clear();
     QRectF bnd=getTotalBoundary(bufferedItems);
@@ -1231,6 +1358,99 @@ void DiagramScene::pasteFromBuffer()
     myMode=CopyingItem;
 }
 
+/*!
+ * \brief to be called when selection has changed
+ * Handle drawing of special elements on top to make all handlers visible
+ */
+void DiagramScene::itemSelectionChangedSlot()
+{
+    if(m_blockSelectionChanged) return;
+    if(m_SelectedItem){
+        // selection changed ?
+        QList<QGraphicsItem*> items=QGraphicsScene::selectedItems();
+        if(items.contains(m_SelectedItem)) {
+            m_blockSelectionChanged=true;
+            DiagramDrawItem *drawItem=qgraphicsitem_cast<DiagramDrawItem*>(m_SelectedItem);
+            if(drawItem){
+                DiagramDrawItem *partner=drawItem->partnerItem();
+                partner->setSelected(true);
+            }
+            DiagramPathItem *pathItem=qgraphicsitem_cast<DiagramPathItem*>(m_SelectedItem);
+            if(pathItem){
+                DiagramPathItem *partner=pathItem->partnerItem();
+                partner->setSelected(true);
+            }
+            DiagramSplineItem *splineItem=qgraphicsitem_cast<DiagramSplineItem*>(m_SelectedItem);
+            if(splineItem){
+                DiagramSplineItem *partner=splineItem->partnerItem();
+                partner->setSelected(true);
+            }
+            this->removeItem(m_SelectedItem);
+            delete m_SelectedItem;
+            m_SelectedItem=nullptr;
+            m_blockSelectionChanged=false;
+        }else{
+            removePartnerItem();
+        }
+
+    }
+    if(!m_SelectedItem){
+        // only when single element is selected
+        QList<QGraphicsItem*> items=QGraphicsScene::selectedItems();
+        if(items.size()==1){
+            QGraphicsItem *item=items.first();
+            if(item->type()==DiagramDrawItem::Type){
+                m_blockSelectionChanged=true;
+                DiagramDrawItem *drawItem=qgraphicsitem_cast<DiagramDrawItem*>(item);
+                DiagramDrawItem *newItem=qgraphicsitem_cast<DiagramDrawItem*>(drawItem->copy());
+                m_SelectedItem = newItem;
+                newItem->setZValue(m_maxZ);
+                m_maxZ+=0.1;
+                addItem(newItem);
+                newItem->setPartnerItem(drawItem);
+                // disable drag on lower level
+                drawItem->setSelected(false);
+                newItem->setFlag(QGraphicsItem::ItemIsMovable, true);
+                newItem->setFlag(QGraphicsItem::ItemIsSelectable, true);
+                newItem->setSelected(true);
+                m_blockSelectionChanged=false;
+            }
+            if(item->type()==DiagramPathItem::Type){
+                m_blockSelectionChanged=true;
+                DiagramPathItem *pathItem=qgraphicsitem_cast<DiagramPathItem*>(item);
+                DiagramPathItem *newItem=qgraphicsitem_cast<DiagramPathItem*>(pathItem->copy());
+                m_SelectedItem = newItem;
+                newItem->setZValue(m_maxZ);
+                m_maxZ+=0.1;
+                addItem(newItem);
+                newItem->setPartnerItem(pathItem);
+                // disable drag on lower level
+                pathItem->setSelected(false);
+                newItem->setFlag(QGraphicsItem::ItemIsMovable, true);
+                newItem->setFlag(QGraphicsItem::ItemIsSelectable, true);
+                newItem->setSelected(true);
+                m_blockSelectionChanged=false;
+            }
+            if(item->type()==DiagramSplineItem::Type){
+                m_blockSelectionChanged=true;
+                DiagramSplineItem *splineItem=qgraphicsitem_cast<DiagramSplineItem*>(item);
+                DiagramSplineItem *newItem=qgraphicsitem_cast<DiagramSplineItem*>(splineItem->copy());
+                m_SelectedItem = newItem;
+                newItem->setZValue(m_maxZ);
+                m_maxZ+=0.1;
+                addItem(newItem);
+                newItem->setPartnerItem(splineItem);
+                // disable drag on lower level
+                splineItem->setSelected(false);
+                newItem->setFlag(QGraphicsItem::ItemIsMovable, true);
+                newItem->setFlag(QGraphicsItem::ItemIsSelectable, true);
+                newItem->setSelected(true);
+                m_blockSelectionChanged=false;
+            }
+        }
+    }
+}
+
 void DiagramScene::setCursorVisible(bool vis)
 {
     if(vis){
@@ -1252,6 +1472,9 @@ void DiagramScene::deleteItem(QGraphicsItem *item)
     if(item==insertedDrawItem){
         insertedDrawItem=nullptr;
     }
+    if(item==m_SelectedItem){
+        m_SelectedItem=nullptr;
+    }
     this->removeItem(item);
 }
 
@@ -1272,10 +1495,10 @@ void DiagramScene::insertElementDirectly(const QString element)
  * active items are selected items or insertItem
  * \return
  */
-QList<QGraphicsItem *> DiagramScene::activeItems() const
+QList<QGraphicsItem *> DiagramScene::activeItems(bool includeSelected)
 {
     if(!selectedItems().isEmpty()){
-        return selectedItems();
+        return selectedItems(includeSelected);
     }
     if(!myMoveItems.isEmpty()){
         return myMoveItems;
@@ -1460,6 +1683,7 @@ void DiagramScene::abort(bool keepSelection)
     insertedDrawItem=nullptr;
     insertedPathItem=nullptr;
     insertedSplineItem=nullptr;
+    m_blockSelectionChanged=false;
     copiedItems.clear();
 
     if(!keepSelection) clearSelection();
@@ -1502,14 +1726,18 @@ bool DiagramScene::load_json(QFile *file)
  * \brief read in json
  * \param doc
  */
-void DiagramScene::read_in_json(QJsonDocument doc)
+QList<QGraphicsItem *> DiagramScene::read_in_json(QJsonDocument doc,bool place)
 {
+    QList<QGraphicsItem*> newItems;
     QJsonArray array=doc.array();
     for(int i=0;i<array.size();++i){
         QJsonObject json=array[i].toObject();
         QGraphicsItem *item=getElementFromJSON(json);
         if(item->zValue()>m_maxZ) m_maxZ=item->zValue();
-        addItem(item);
+        if(place){
+            addItem(item);
+        }
+        newItems.append(item);
         if(item->type()==DiagramItem::Type){
             QRectF rect;
             for(const auto* it:item->childItems()){
@@ -1525,6 +1753,7 @@ void DiagramScene::read_in_json(QJsonDocument doc)
     insertedSplineItem = nullptr;
     textItem = nullptr;
     myMode = MoveItem;
+    return newItems;
 }
 /*!
  * \brief add item as json to JSON array
@@ -1623,7 +1852,7 @@ QGraphicsItem *DiagramScene::getElementFromJSON(QJsonObject json)
         connect(textItem, &DiagramTextItem::receivedFocus,
                 this, &DiagramScene::editorReceivedFocus);
         connect(textItem, &DiagramTextItem::selectedChange,
-                this, &DiagramScene::itemSelected);
+                this, &DiagramScene::textItemSelected);
         item=textItem;
         break;
     case QGraphicsItemGroup::Type:
