@@ -57,7 +57,7 @@ MainWindow::MainWindow(int argc, char *argv[], QWidget *parent)
     m_scene = new DiagramScene(itemMenu, this);
     m_scene->setSceneRect(QRectF(0, 0, 5000, 5000));
     m_scene->setGridVisible(configuration.showGrid);
-    connect(m_scene, &DiagramScene::itemSelected,
+    connect(m_scene, &DiagramScene::textItemSelected,
             this, &MainWindow::itemSelected);
     connect(m_scene, &DiagramScene::forceCursor,
             this, &MainWindow::moveCursor);
@@ -135,6 +135,7 @@ void MainWindow::redo()
  */
 void MainWindow::closeEvent(QCloseEvent *event)
 {
+    m_scene->clearSelection(); // avoid crash
     QSettings settings("QDia", "QDia");
     settings.setValue("geometry", saveGeometry());
     settings.setValue("windowState", saveState());
@@ -188,7 +189,7 @@ void MainWindow::buttonGroupClicked(QAbstractButton *button)
 
 void MainWindow::deleteItem()
 {
-    QList<QGraphicsItem *> selectedItems = m_scene->selectedItems();
+    QList<QGraphicsItem *> selectedItems = m_scene->selectedItems(true);
 
     for (int i=0;i<selectedItems.length();++i) {
         QGraphicsItem *it=selectedItems[i];
@@ -427,7 +428,7 @@ void MainWindow::rotateRight()
     if (m_scene->activeItems().isEmpty())
         return;
 
-    transformSelected(QTransform().rotate(90),m_scene->activeItems(),true);
+    transformSelected(QTransform().rotate(90),m_scene->activeItems(true),true);
 }
 
 void MainWindow::rotateLeft()
@@ -435,7 +436,7 @@ void MainWindow::rotateLeft()
     if (m_scene->activeItems().isEmpty())
         return;
 
-    transformSelected(QTransform().rotate(-90),m_scene->activeItems(),true);
+    transformSelected(QTransform().rotate(-90),m_scene->activeItems(true),true);
 }
 
 void MainWindow::flipX()
@@ -443,7 +444,7 @@ void MainWindow::flipX()
     if (m_scene->activeItems().isEmpty())
         return;
 
-    transformSelected(QTransform(-1,0,0,1,0,0),m_scene->activeItems());
+    transformSelected(QTransform(-1,0,0,1,0,0),m_scene->activeItems(true));
 }
 
 void MainWindow::flipY()
@@ -451,7 +452,7 @@ void MainWindow::flipY()
     if (m_scene->activeItems().isEmpty())
         return;
 
-    transformSelected(QTransform(1,0,0,-1,0,0),m_scene->activeItems());
+    transformSelected(QTransform(1,0,0,-1,0,0),m_scene->activeItems(true));
 }
 
 void MainWindow::scale()
@@ -466,7 +467,7 @@ void MainWindow::scale()
     if (!ok)
         return;
 
-    transformSelected(QTransform().scale(scaleFactor,scaleFactor),m_scene->activeItems(),true);
+    transformSelected(QTransform().scale(scaleFactor,scaleFactor),m_scene->activeItems(true),true);
 }
 
 void MainWindow::currentFontChanged(const QFont &)
@@ -621,6 +622,7 @@ void MainWindow::createToolBox()
     // added DrawItem
     QList<Element> elements{
         {tr("Rectangle"),DiagramDrawItem::Rectangle},
+        {tr("Square"),DiagramDrawItem::Square},
         {tr("Ellipse"),DiagramDrawItem::Ellipse},
         {tr("Circle"),DiagramDrawItem::Circle},
         {tr("RoundedRect"),DiagramDrawItem::RoundedRect},
@@ -1467,10 +1469,14 @@ void MainWindow::copyToClipboard()
     // TODO: only selected whould be exported to clipboard
     m_scene->setCursorVisible(false);
     m_scene->copyToBuffer();
+    // save selected items to bytearray
+    QJsonDocument doc=m_scene->create_json_save(true);
+    QByteArray buffer(doc.toJson(QJsonDocument::Compact));
+    // unselect everything
     m_scene->abort();
     bool gridVisible=m_scene->isGridVisible();
     m_scene->setGridVisible(false);
-    QClipboard *clipboard = QGuiApplication::clipboard();
+
     QRectF rect=m_scene->itemsBoundingRect();
     rect.adjust(-1,-1,1,1);
     qreal w=rect.width();
@@ -1492,14 +1498,29 @@ void MainWindow::copyToClipboard()
     painter.setRenderHint(QPainter::Antialiasing);
     m_scene->render(&painter,QRectF(),rect);
     painter.end();
-    clipboard->setPixmap(pixmap);
+    // copy image into mimedata
+    QMimeData *mimeData = new QMimeData;
+    mimeData->setImageData(pixmap);
+    // add qdia data to mimedata
+    mimeData->setData("application/qdia",buffer);
+    mimeData->setData("application/qdia_id",m_id);
+    // copy mimedata to clipboard
+    QClipboard *clipboard = QGuiApplication::clipboard();
+    clipboard->setMimeData(mimeData);
     m_scene->setCursorVisible(true);
     m_scene->setGridVisible(gridVisible);
 }
 
 void MainWindow::pasteFromClipboard()
 {
-    m_scene->pasteFromBuffer();
+    // check if we have a qdia mimedata
+    QClipboard *clipboard = QGuiApplication::clipboard();
+    const QMimeData *mimeData = clipboard->mimeData();
+    QByteArray buffer;
+    if (!clipboard->ownsClipboard() && mimeData->hasFormat("application/qdia")) {
+        buffer = mimeData->data("application/qdia");
+    }
+    m_scene->pasteFromBuffer(buffer);
 }
 
 void MainWindow::groupItems()
@@ -2298,6 +2319,31 @@ void MainWindow::transformSelected(const QTransform transform, QList<QGraphicsIt
  */
 void MainWindow::transformItems(const QTransform transform, QList<QGraphicsItem *> items, QPointF anchorPoint)
 {
+    // special treatment for selected draw/path/spline item (extra copy to show on top)
+    if(items.size()==1){
+        QGraphicsItem *item=items.first();
+        if(item->type()==DiagramPathItem::Type){
+            DiagramPathItem *it=qgraphicsitem_cast<DiagramPathItem*>(item);
+            if(it){
+                auto *partner=it->partnerItem();
+                items.append(partner);
+            }
+        }
+        if(item->type()==DiagramSplineItem::Type){
+            DiagramSplineItem *it=qgraphicsitem_cast<DiagramSplineItem*>(item);
+            auto *partner=it->partnerItem();
+            if(partner){
+                items.append(partner);
+            }
+        }
+        if(item->type()==DiagramDrawItem::Type){
+            DiagramDrawItem *it=qgraphicsitem_cast<DiagramDrawItem*>(item);
+            auto *partner=it->partnerItem();
+            if(partner){
+                items.append(partner);
+            }
+        }
+    }
     foreach( QGraphicsItem *item, items){
         if(!item) continue;
         // special treatment for texts
@@ -2384,8 +2430,12 @@ void MainWindow::linePatternChanged()
 }
 
 /* TODO
- ** color template ? / store custom colors
- ** show handlers on top when selected and below
+ * color template ? / store custom colors
+ * click unselect (shift/? alt)
+ * click again to select lower element
+ * better handlers when zoomed out ?
+ * cut line
+ * manhattan routing
  * export wider to entail wider lines ?
  * user elements -> order ?
  * fix flip/rotate when moving/dragging several elements
@@ -2393,6 +2443,6 @@ void MainWindow::linePatternChanged()
  * tap style
  * Align ?
  * import xcircuit/drawio?
- ** read xcircuit lps
- ** show svg
+ * read xcircuit lps
+ * show svg
  */
